; ------------------------------
; THE improved 'ARCSIN' FUNCTION
; ------------------------------
; (Offset $1F: 'asn')
;   The inverse sine function with result in radians.
;   Derived from arctan function above.
;   Error A unless the argument is between -1 and +1 inclusive.
;   Uses an adaptation of the formula asn(x) = atn(x/sqr(1-x*x))
;
;	    /|
;	   / |
;	 1/  |x
;	 /a  |
;	/----|    
;	  y
;
;   e.g. We know the opposite side (x) and hypotenuse (1) 
;   and we wish to find angle a in radians.
;   We can derive length y by Pythagoras and then use ATN instead. 
;   Since y*y + x*x = 1*1 (Pythagoras Theorem) then y=sqr(1-x*x)
;   - no need to multiply 1 by itself.
;   So, asn(a) = atn(x/y)
;   or more fully,
;   asn(a) = atn(x/sqr(1-x*x))

;   Close but no cigar.

;   While PRINT ATN (x/SQR (1-x*x)) gives the same results as PRINT ASN x,
;   it leads to division by zero when x is 1 or -1.
;   To overcome this, 1 is added to y giving half the required angle and the 
;   result is then doubled. 
;   That is, PRINT ATN (x/(SQR (1-x*x) +1)) *2
;
;
;		. /|
;	     .  c/ |
;	  .     /1 |x
;      . c   b /a  |
;    ---------/----|    
;      1      y
;
;   By creating an isosceles triangle with two equal sides of 1, angles c and 
;   c are also equal. If b+c+c = 180 degrees and b+a = 180 degress then c=a/2.
;
;   A value higher than 1 gives the required error as attempting to find  the
;   square root of a negative number generates an error in Sinclair BASIC.
;
fn_asn
	RST 28H		;; FP-CALC		x.

;;-	DEFB $2D	;;duplicate		x, x.
;;-	DEFB $2D	;;duplicate		x, x, x.
;;-	DEFB $04	;;multiply		x, x*x.

	.db $3D		;;stk-square		x, x*x.

;;-	DEFB $A1	;;stk-one		x, x*x, 1.
;;-	DEFB $03	;;subtract		x, x*x-1.

	.db $39		;;sub-one macro		x, x*x-1.

	DEFB $18	;;negate		x, 1-x*x.
	DEFB $25	;;sqr			x, sqr(1-x*x) = y.
	DEFB $A1	;;stk-one		x, y, 1.
	DEFB $0F	;;addition		x, y+1.
	DEFB $05	;;division		x/(y+1).
	DEFB $21	;;atn			a/2	(half the angle)

;;-	DEFB $2D	;;duplicate		a/2, a/2.
;;-	DEFB $0F	;;addition		a.

	DEFB $34	;;end-calc		return via mul-by-2:  a=2*a/2.

;;-	RET		 	; return.
;
; ------------------------------
; THE 'MULTIPLY BY TWO' FUNCTION
; ------------------------------
; (Offset $3A: 'mul-by-2')
; this part is a new 'macro', which replaces the next 2 calc. literals
;
;	$2D  :  duplicate	(x,x)
;	$0F  :  addition	(x+x = 2*x)
;
mul_by_2
