tbl_offs .equ $-tbl_addrs
;
;   the following are just the next available slots for the 128 compound 
;   literals which are in range $80 - $FF.
;
	DEFW  seriesg_x		; series-xx    $80 - $9F.
	DEFW  stk_con_x		; stk-const-xx $A0 - $BF.
	DEFW  sto_mem_x		; st-mem-xx    $C0 - $DF.
	DEFW  get_mem_x		; get-mem-xx   $E0 - $FF.
;
; Aside: 42 - 7F are therefore unused calculator literals.
;	 3C - 7B would be available for expansion.
;
; ----------------------------------------
; THE improved 'FLOATING POINT CALCULATOR'
; ----------------------------------------
;
CALCULATE
	CALL STK_PNTRS		; routine STK-PNTRS is called to set up the
				; calculator stack pointers for a default
				; unary operation. HL = last value on stack.
				; DE = STKEND first location after stack.
;
;   the calculate routine is called at this point by the series generator...
;
GEN_ENT_1
	LD    A,B		; fetch the Z80 B register to A
	LD    ($401E),A		; and store value in system variable BREG.
				; this will be the counter for dec-jr-nz
				; or if used from fp-calc2 the calculator
				; instruction.
;
;   ... and again later at this point
;
GEN_ENT_2
	EXX			; switch sets
	EX (SP),HL		; and store the address of next instruction,
				; the return address, in H'L'.
				; If this is a recursive call then the H'L'
				; of the previous invocation goes on stack.
				; c.f. end-calc.
	EXX			; switch back to main set.
;
;   this is the re-entry looping point when handling a string of literals.
;
RE_ENTRY
	LD ($401C),DE		; save end of stack in system variable STKEND
	EXX			; switch to alt
	LD A,(HL)		; get next literal
	INC HL			; increase pointer'
;
;   single operation jumps back to here
;
SCAN_ENT
	PUSH  HL		; save pointer on stack   *
	AND A			; now test the literal
;;-	JP P,FIRST_3D		; forward to FIRST-3D if in range $00 - $3D

	JP P,FIRST_7F		; forward to FIRST-7F if in range $00 - $7F
				; anything with bit 7 set will be one of
				; 128 compound literals.
;
;   Compound literals have the following format.
;   bit 7 set indicates compound.
;   bits 6-5 the subgroup 0-3.
;   bits 4-0 the embedded parameter $00 - $1F.
;   The subgroup 0-3 needs to be manipulated to form the next available four
;   address places after the simple literals in the address table.
;
	LD D,A			; save literal in D
	AND $60			; and with 01100000 to isolate subgroup
	RRCA			; rotate bits
	RRCA			; 4 places to right
	RRCA			; not five as we need offset * 2
	RRCA			; 00000xx0

;;-	ADD A,$72		; add ($39 * 2) to give correct offset.
;;-				; alter above if you add more literals.

	ADD A,tbl_offs		; correct offset.
	LD L,A			; store in L for later indexing.
	LD A,D			; bring back compound literal
	AND $1F			; use mask to isolate parameter bits
	JR ENT_TABLE		; forward to ENT-TABLE
; ---
;
;   the branch was here with simple literals.
;
;;- FIRST_3D
FIRST_7F
	CP $18			; compare with first unary operations.
	JR NC,DOUBLE_A		; to DOUBLE-A with unary operations
;
;   it is binary so adjust pointers.
;
	EXX			;

;;-	LD BC,$FFFB		; the value -5
;;-	LD D,H			; transfer HL, the last value, to DE.
;;-	LD E,L			;
;;-	ADD HL,BC		; subtract 5 making HL point to second
				; value.
	ex de,hl		; transfer HL, the last value, to DE.
	ld hl,$FFFB		; the value -5
	add hl,de		; subtract 5 making HL point to second
				; value.
	EXX			;
DOUBLE_A
	RLCA			; double the literal
	LD L,A			; and store in L for indexing
ENT_TABLE
	LD DE,tbl_addrs		; Address: tbl-addrs
	LD H,$00		; prepare to index
	ADD HL,DE		; add to get address of routine
	LD E,(HL)		; low byte to E
	INC HL			;
	LD D,(HL)		; high byte to D

	LD HL,RE_ENTRY		; Address: RE-ENTRY
	EX (SP),HL		; goes on machine stack
				; address of next literal goes to HL. *

	PUSH DE			; now the address of routine is stacked.
	EXX			; back to main set
				; avoid using IY register.
	LD BC,($401D)		; STKEND_hi
				; nothing much goes to C but BREG to B
				; and continue into next ret instruction
				; which has a dual identity
