; ---------------------------------------------------------
; THE improved 'FLOATING-POINT TO BC' SUBROUTINE (67 bytes)
; ---------------------------------------------------------
; The floating-point form on the calculator stack is compressed
; directly into the BC register rounding up if necessary.
; Valid range is 0 to 65535.4999
;
;; FP-TO-BC
L158A:	CALL STK_FETCH		; routine STK-FETCH - exponent to A
				; mantissa to EDCB.
	rla			; test if abs(x)>=0.5  (exp>=128)
	jr c,L1595		; forward if yes (CY=1) to FPBC-NZRO
;
; else value is zero
;
	xor a			; else clear CY and A, set Z flag
	LD B,A			; zero to B
	LD C,A			; also to C
	jr L15C4		; forward to FPBC-ZRO
; ---
;
; EDCB  =>  BCE
;
;; FPBC-NZRO
L1595:
	LD B,E			; transfer the mantissa from EDCB
	LD E,C			; to BCE. Bit 7 of E is the 17th bit which
	LD C,D			; will be significant for rounding if the
				; number is already normalized.

	rra			; restore the original exponent

	SUB $91			; subtract 65536
	CCF			; complement carry flag
	BIT 7,B			; test sign bit
	PUSH AF			; push the result

	SET 7,B			; set the implied bit
	JR C,L15C5		; forward with carry from SUB/CCF to FPBC-END
				; number is too big.

	cpl			; complement to make range $00 - $0F

	CP $08			; test if one or two bytes
	JR C,L15AE		; forward with two to BIG-INT

	LD E,C			; shift mantissa
	LD C,B			; 8 places right
	LD B,$00		; insert a zero in B
	SUB $08			; reduce exponent by eight
;
;; BIG-INT
L15AE:	AND A			; test the exponent
	LD D,A			; save exponent in D.

	LD A,E			; fractional bits to A
	RLCA			; rotate most significant bit to carry for
				; rounding of an already normal number.

	JR Z,L15BB		; forward if exponent zero to EXP-ZERO
				; the number is normalized
;
;; FPBC-NORM
L15B4:	SRL B			;   0->76543210->C
	RR C			;   C->76543210->C

	DEC D			; decrement exponent
	JR NZ,L15B4		; loop back till zero to FPBC-NORM
;
;; EXP-ZERO
L15BB:
	JR NC,L15C5		; forward without carry to FPBC-END (NO-ROUND)

	INC BC			; round up.
	LD A,B			; test result
	OR C			; for zero
	JR NZ,L15C5		; forward if not to FPBC-END

	POP AF			; restore sign flag
	SCF			; set carry flag to indicate overflow

;; FPBC-ZRO
L15C4:	PUSH AF			; save combined flags again
;
;; FPBC-END
L15C5:	PUSH BC			; save BC value

; set HL and DE to calculator stack pointers.

;;-	RST 28H		;; FP-CALC
;;-	DEFB $34	;;end-calc

	call STK_PNTRS		; routine STK-PNTRS is called to set up the
				; calculator stack pointers for a default
				; unary operation. HL = last value on stack.
				; DE = STKEND first location after stack.

	POP BC			; restore BC value
	POP AF			; restore flags
	LD A,C			; copy low byte to A also.
	RET 			; return
;
; ------------------------------------
;
;;-	.db $FF			; spare
