; ------------------------------------------
; THE improved 'SERIES GENERATOR' SUBROUTINE
; ------------------------------------------
; offset $86: 'series-06'
; offset $88: 'series-08'
; offset $8C: 'series-0C'
; The ZX81 uses Chebyshev polynomials to generate approximations for
; SIN, ATN, LN and EXP. These are named after the Russian mathematician
; Pafnuty Chebyshev, born in 1821, who did much pioneering work on numerical
; series. As far as calculators are concerned, Chebyshev polynomials have an
; advantage over other series, for example the Taylor series, as they can
; reach an approximation in just six iterations for SIN, eight for EXP and
; twelve for LN and ATN. The mechanics of the routine are interesting but
; for full treatment of how these are generated with demonstrations in
; Sinclair BASIC see "The Complete Spectrum ROM Disassembly" by Dr Ian Logan
; and Dr Frank O'Hara, published 1983 by Melbourne House.
;
seriesg_x
	LD B,A			; parameter $00 - $1F to B counter
	CALL GEN_ENT_1		; routine GEN-ENT-1 is called.
				; A recursive call to a special entry point
				; in the calculator that puts the B register
				; in the system variable BREG. The return
				; address is the next location and where
				; the calculator will expect its first
				; instruction - now pointed to by HL'.
				; The previous pointer to the series of
				; five-byte numbers goes on the machine stack.
;
; The initialization phase.
;
;;-	DEFB $2D	;;duplicate		x,x
;;-	DEFB $0F	;;addition		x+x

	.db $3A		;;mul-by-2		2*x

	DEFB $C0	;;st-mem-0		2*x
	DEFB $02	;;delete		.
	DEFB $A0	;;stk-zero		0
	DEFB $C2	;;st-mem-2		0
;
; a loop is now entered to perform the algebraic calculation for each of
; the numbers in the series
;
G_LOOP
	DEFB $2D	;;duplicate		v,v.
	DEFB $E0	;;get-mem-0		v,v,2*x
	DEFB $04	;;multiply		v,v*2*x
	DEFB $E2	;;get-mem-2		v,v*2*x,v
	DEFB $C1	;;st-mem-1		v,v*2*x,v
	DEFB $03	;;subtract		v,v*2*x-v
	DEFB $34	;;end-calc
;
; the previous pointer is fetched from the machine stack to H'L' where it
; addresses one of the numbers of the series following the series literal.
;
	CALL  stk_data		; routine STK-DATA is called directly to
				; push a value and advance H'L'.
	CALL  GEN_ENT_2		; routine GEN-ENT-2 recursively re-enters
				; the calculator without disturbing
				; system variable BREG
				; H'L' value goes on the machine stack and is
				; then loaded as usual with the next address.

	DEFB $0F	;;addition
	DEFB $01	;;exchange
	DEFB $C2	;;st-mem-2
	DEFB $02	;;delete

	DEFB $31	;;dec-jr-nz
	DEFB G_LOOP-$	;;back to G-LOOP

; when the counted loop is complete the final subtraction yields the result
; for example SIN X.

	DEFB $E1	;;get-mem-1
	DEFB $03	;;subtract
	DEFB $34	;;end-calc

	RET			; return with H'L' pointing to location
				; after last number in series.
