; ----------------------------
; THE old 'TO POWER' OPERATION
; ----------------------------
; (Offset $06: 'to-power')
;   The 'Exponential' operation.
;   This raises the first number X to the power of the second number Y.
;   e.g. PRINT 2 ** 3 gives the result 8
;   As with the ZX80,
;   0 ** 0 = 1
;   0 ** +n = 0
;   0 ** -n = arithmetic overflow.

to_power
	RST 28H		;; FP-CALC		X,Y.

	.db $01		;;exchange		Y,X.
	.db $2D		;;duplicate		Y,X,X.

	.db $2C		;;not			Y,X,(1/0).
	.db $00		;;jump-true
	.db X_IS_0-$	;;forward to X_IS_0	if X is zero.
;
;   else X is non-zero. function 'ln' will catch a negative value of X.
;
	.db $22		;;ln			Y, LN X.
;
;   Multiply the power by the logarithm of the argument.
;
	.db $04		;;multiply		Y * LN X
	.db $34		;;end-calc

	JP exp			; jump back to EXP routine  ->> 
				; to find the 'antiln'
; ---
;
;   these routines form the three simple results when the number is zero.
;   begin by deleting the known zero to leave Y the power factor.
;
X_IS_0
	.db $02		;;delete		Y.
	.db $2D		;;duplicate		Y, Y.

	.db $2C		;;not			Y, (1/0).
	.db $00		;;jump-true	
	.db ONE-$	;;forward to ONE	if Y is zero.
;
;   the power factor is not zero. If negative then an error exists.
;
	.db $A0		;;stk-zero		Y, 0.
	.db $01		;;exchange		0, Y.

	.db $33		;;greater-0		0, (1/0).
	.db $00		;;jump-true		0
	.db LAST-$	;;to LAST		if Y was any positive number.

;   else force division by zero thereby raising an Arithmetic overflow error.
;   As an alternative, this now raises an error directly.

	.db $34		;; end-calc
; ---
REPORT_6c
	RST 08H			;+ ERROR-1
	.db $05			;+ Error Report: Number too big
; ---
ONE
	.db $02		;;delete		.
	.db $A1		;;stk-one		1.
LAST
	.db $34		;;end-calc		last value 1 or 0.

	RET 		; return.
;
; ===========================================================================
;
; -----------------------------------------
; THE improved 'TO POWER' OPERATION (cont.)
; -----------------------------------------
;
;   X is non-zero. function 'ln' will catch a negative value of X.
to_pwr2
	rst 28h		;; FP-CALC		X,Y.
	.db $01		;;exchange		Y, X.
	.db $22		;;ln			Y, LN X.
;
;   Multiply the power by the logarithm of the argument.
;
	.db $04		;;multiply		Y * LN X
	.db $34		;;end-calc
;
;   Then find the 'antiln'
;
; --------------------------
; THE 'EXPONENTIAL' FUNCTION
; --------------------------
exp
;
;	.
;	.
;	.


; ---------------------------------
; THE improved 'TO POWER' OPERATION
; ---------------------------------
; (Offset $06: 'to-power')
;   The 'Exponential' operation.
;   This raises the first number X to the power of the second number Y.
;   e.g. PRINT 2 ** 3 gives the result 8
;   As with the ZX80,
;   0 ** 0 = 1
;   0 ** +n = 0
;   0 ** -n = arithmetic overflow.
;
to_power			; HL points X, DE points Y.
	ld a,(hl)		; fetch X.exp
	and a			; test zero
	jp nz,to_pwr2		; continue if X<>0
; X_IS_0
	ex de,hl		; else switch pointers
	inc hl			; 
	bit 7,(hl)		; test if Y is negative
	dec hl			; 
	ld a,(hl)		; fetch Y.exp
	ex de,hl		; switch back pointers
	jp z,not_1		; jump if it is positive or zero to
				; replace X with 1 (Y=0) or 0 (Y<>0)
; ---
	rst 08h			; else Error Report:
	.db $05			; arithmetic overflow


