;
; ------------------------
; THE 'TABLE OF ADDRESSES'
; ------------------------
;
; starts with binary operations which have two operands and one result.
; three pseudo binary operations first.
;
tbl_addrs
	DEFW  jump_true		; $00 Address: $1C2F - jump-true
;	.
;	.
;	.
	DEFW  e_to_fp		; $38 Address: $155A - e-to-fp
;
;   new macros
;
	.dw sub_one		; $39 macro sub-one (part of 'INT')
	.dw mul_by_2		; $3A macro mul-by-2
	.dw mul_by10		; $3B macro mul-by-10
	.dw stk_squa		; $3C macro stk-square
;
tbl_offs .equ $-tbl_addrs
;
;   the following are just the next available slots for the 128 compound 
;   literals which are in range $80 - $FF.
;
	DEFW  seriesg_x		; series-xx    $80 - $9F.
	DEFW  stk_con_x		; stk-const-xx $A0 - $BF.
	DEFW  sto_mem_x		; st-mem-xx    $C0 - $DF.
	DEFW  get_mem_x		; get-mem-xx   $E0 - $FF.
;
; Aside: 41 - 7F are therefore unused calculator literals.
;	 3D - 7B would be available for expansion.

;	.
;	.
;	.

;
; ------------------------------
; THE 'MULTIPLY BY TWO' FUNCTION
; ------------------------------
; (Offset $3A: 'mul-by-2')
; this part is a new 'macro', which replaces the next 2 calc. literals
;
;	$2D  :  duplicate	(x,x)
;	$0F  :  addition	(x+x = 2*x)
;
mul_by_2
	ld a,(hl)		; test exponent
	and a			; =0?
	ret z			; return if it is zero (2*0=0!)

	inc (hl)		; else increase the exponent (*2)
	ret nz			; return if no overflow occurred
; ---
	rst 08h			; Error Report:
	.db $05			; Number is too big
;
; ------------------------------
; THE 'MULTIPLY BY TEN' FUNCTION
; ------------------------------
; (Offset $3C: 'mul-by-10')
; the 'tricky' multiplication:  10x = 2x + 8x
; the new E-TO-FP and PRINT-FP call this subroutine
;
mul_by10
	call mul_by_2		; x1 = x*2 	(tests overflow)
	ret z			; return if it is zero (2*0=0!)

	push hl			; save OP1 pointer
	call COPY_FP		; duplicate 

	call mul_by_2		; x2 = x1*2 	(tests overflow)
	call mul_by_2		; x3 = x1*4 	(tests overflow)

	ex de,hl		; de: OP2 pointer
	pop hl			; hl: OP1 pointer
	jp addition		; routine addition (L1755)
				; y = 2*x + 8*x    (=10*x)
;
; ---------------------------
; THE 'STACK-SQUARE' FUNCTION
; ---------------------------
; (Offset $3D: 'stk-square') 
; this part is a new 'macro', which replaces the next 3 calc. literals
;
;	$2D  :  duplicate	(x, x)
;	$2D  :  duplicate	(x, x, x)
;	$04  :  multiply	(x, x*x)
;
stk_squa
	call COPY_FP		; duplicate 

	push de			; save stack end
	ld d,h			; set pointer to
	ld e,l			; last value on stack
	call multiply		; multiplication

	pop de			; restore pointer
	ret			; return w. square
